/* Foundation v2.2 http://foundation.zurb.com */
/* Artfully Masterminded by ZURB */

/* --------------------------------------------------
    Table of Contents
-----------------------------------------------------
:: Reset & Standards
:: Links
:: Lists
:: Tables
:: Misc
*/


/*	--------------------------------------------------
	:: Global Reset & Standards
	-------------------------------------------------- */

	/*
		Eric Meyer's CSS Reset
		http://meyerweb.com/eric/tools/css/reset/
		v2.0 | 20110126
   		License: none (public domain)
	*/

	html, body, div, span, applet, object, iframe,
	h1, h2, h3, h4, h5, h6, p, blockquote, pre,
	a, abbr, acronym, address, big, cite, code,
	del, dfn, em, img, ins, kbd, q, s, samp,
	small, strike, strong, sub, sup, tt, var,
	b, u, i, center,
	dl, dt, dd, ol, ul, li,
	fieldset, form, label, legend,
	table, caption, tbody, tfoot, thead, tr, th, td,
	article, aside, canvas, details, embed,
	figure, figcaption, footer, header, hgroup,
	menu, nav, output, ruby, section, summary,
	time, mark, audio, video {
		margin: 0;
		padding: 0;
		border: 0;
		font: inherit;
		vertical-align: baseline;
	}
	html {
		font-size: 62.5%;
	}
	/* HTML5 display-role reset for older browsers */
	article, aside, details, figcaption, figure,
	footer, header, hgroup, menu, nav, section {
		display: block;
	}
	body {
		line-height: 1;
	}
	ol, ul {
		list-style: none;
	}
	blockquote, q {
		quotes: none;
	}
	blockquote:before, blockquote:after,
	q:before, q:after {
		content: '';
		content: none;
	}
	table {
		border-collapse: collapse;
		border-spacing: 0;
	}

	body { background: #fff; font-family: "Helvetica Neue", "HelveticaNeue", Helvetica, Arial, "Lucida Grande", sans-serif; font-size: 13px; line-height: 18px; color: #555; position: relative; -webkit-font-smoothing: antialiased; }



/*	--------------------------------------------------
	:: Links
	-------------------------------------------------- */
	a { color: #2a85e8; text-decoration: none; line-height: inherit; }
	a:hover { color: #11639d; }
	a:focus { color: #cc4714; outline: none; }
	p a, p a:visited { line-height: inherit; }


/*	--------------------------------------------------
	:: Lists
	-------------------------------------------------- */
	ul, ol { margin-bottom: 18px; }
	ul { list-style: none outside; }
	ol { list-style: decimal; }
	ol, ul.square, ul.circle, ul.disc { margin-left: 30px; }
	ul.square { list-style: square outside; }
	ul.circle { list-style: circle outside; }
	ul.disc { list-style: disc outside; }
	li { margin-bottom: 12px; }
	ul.large li { line-height: 21px; }

/*	--------------------------------------------------
	:: Tables
	-------------------------------------------------- */
	table.fnd_table { background: #fff; -moz-border-radius: 3px; -webkit-border-radius: 3px; border-radius: 3px; margin: 0 0 18px; border: 1px solid #ddd;  }

	table.fnd_table thead, table.fnd_table tfoot { background: #f5f5f5; }
	table.fnd_table thead tr th,
	table.fnd_table tfoot tr th,
	table.fnd_table tbody tr td,
	table.fnd_table tr td,
	table.fnd_table tfoot tr td { font-size: 12px; line-height: 18px; text-align: left; }
	table.fnd_table thead tr th,
	table.fnd_table tfoot tr td { padding: 8px 10px 9px; font-size: 14px; font-weight: bold; color: #222; }
	table.fnd_table thead tr th:first-child, table.fnd_table tfoot tr td:first-child { border-left: none; }
	table.fnd_table thead tr th:last-child, table.fnd_table tfoot tr td:last-child { border-right: none; }

	table.fnd_table tbody tr.even,
	table.fnd_table tbody tr.alt { background: #f9f9f9; }
	table.fnd_table tbody tr:nth-child(even) { background: #f9f9f9; }
	table.fnd_table tbody tr td { color: #333; padding: 9px 10px; vertical-align: top; border: none; }

/* --------------------------------------------------
	:: Misc
---------------------------------------------------*/
	.left        { float: left; }
	.right       { float: right; }
	.text-left   { text-align: left; }
	.text-right  { text-align: right; }
	.text-center { text-align: center; }
	.hide        { display: none; }
	.highlight   { background: #ff0; }

/*	--------------------------------------------------
    :: Typography
	-------------------------------------------------- */
	h1, h2, h3, h4, h5, h6 { color: #181818; font-weight: bold; line-height: 1.25 }
	h1 a, h2 a, h3 a, h4 a, h5 a, h6 a { font-weight: inherit; }
	h1 { font-size: 46px; font-size: 4.6rem; margin-bottom: 12px;}
	h2 { font-size: 35px; font-size: 3.5rem; margin-bottom: 9px; }
	h3 { font-size: 28px; font-size: 2.8rem; margin-bottom: 9px; }
	h4 { font-size: 21px; font-size: 2.1rem; margin-bottom: 3px; }
	h5 { font-size: 18px; font-size: 1.8rem; font-weight: normal; margin-bottom: 3px;  }
	h6 { font-size: 15px; font-size: 1.5rem; font-weight: normal; }

	.subheader { color: #777; font-weight: 300; margin-bottom: 24px; }

	p { font-size: 13px; font-size: 1.3rem; line-height: 1.25; margin: 0 0 18px; }
	p img { margin: 0; }
	p.lead { font-size: 18px; font-size: 1.8rem; line-height: 1.5;  }

	em, i { font-style: italic; line-height: inherit; }
	strong, b { font-weight: bold; line-height: inherit; }
	small { font-size: 60%; line-height: inherit; }

	h1 small, h2 small, h3 small, h4 small, h5 small { color: #777; }

	/*	Blockquotes  */
	blockquote, blockquote p { line-height: 20px; color: #777; }
	blockquote { margin: 0 0 18px; padding: 9px 20px 0 19px; border-left: 1px solid #ddd; }
	blockquote cite { display: block; font-size: 12px; font-size: 1.2rem; color: #555; }
	blockquote cite:before { content: "\2014 \0020"; }
	blockquote cite a, blockquote cite a:visited { color: #555; }

	hr { border: solid #ddd; border-width: 1px 0 0; clear: both; margin: 12px 0 18px; height: 0; }

	abbr, acronym { text-transform: uppercase; font-size: 90%; color: #222; border-bottom: 1px solid #ddd; cursor: help; }
	abbr { text-transform: none; }

	/**
 	 * Print styles.
	 *
	 * Inlined to avoid required HTTP connection: www.phpied.com/delay-loading-your-print-css/
	 * Credit to Paul Irish and HTML5 Boilerplate (html5boilerplate.com)
	 */
	.print-only { display: none !important; }
	@media print {
		* { background: transparent !important; color: black !important; text-shadow: none !important; filter:none !important;
		-ms-filter: none !important; } /* Black prints faster: sanbeiji.com/archives/953 */
		p a, p a:visited { color: #444 !important; text-decoration: underline; }
		p a[href]:after { content: " (" attr(href) ")"; }
		abbr[title]:after { content: " (" attr(title) ")"; }
		.ir a:after, a[href^="javascript:"]:after, a[href^="#"]:after { content: ""; }  /* Don't show links for images, or javascript/internal links */
		pre, blockquote { border: 1px solid #999; page-break-inside: avoid; }
		thead { display: table-header-group; } /* css-discuss.incutio.com/wiki/Printing_Tables */
		tr, img { page-break-inside: avoid; }
		@page { margin: 0.5cm; }
		p, h2, h3 { orphans: 3; widows: 3; }
		h2, h3{ page-break-after: avoid; }
		.hide-on-print { display: none !important; }
		.print-only { display: block !important; }
	}

/* --------------------------------------------------
	:: Grid

	This is the mobile-friendly, responsive grid that
	lets Foundation work much of its magic.

	-------------------------------------------------- */

	.container { padding: 0 20px; }

	.row { width: 100%; max-width: 980px; min-width: 727px; margin: 0 auto; }
	/* To fix the grid into a certain size, set max-width to width */
	.row .row { min-width: 0; }

	.column, .columns { margin-left: 4.4%; float: left; min-height: 1px; position: relative; }
	.column:first-child, .columns:first-child { margin-left: 0; }
	[class*="column"] + [class*="column"]:last-child { float: right; }

	.row .one 		{ width: 4.3%; }
	.row .two 		{ width: 13%; }
	.row .three 	{ width: 21.679%; }
	.row .four 		{ width: 30.37%; }
	.row .five 		{ width: 39.1%; }
	.row .six 		{ width: 47.8%; }
	.row .seven 	{ width: 56.5%; }
	.row .eight 	{ width: 65.2%; }
	.row .nine 		{ width: 73.9%; }
	.row .ten 		{ width: 82.6%; }
	.row .eleven 	{ width: 91.3%; }
	.row .twelve 	{ width: 100%; }

	.row .offset-by-one 	{ margin-left: 13.1%;  }
	.row .offset-by-two 	{ margin-left: 21.8%;  }
	.row .offset-by-three 	{ margin-left: 30.5%;  }
	.row .offset-by-four 	{ margin-left: 39.2%;  }
	.row .offset-by-five 	{ margin-left: 47.9%;  }
	.row .offset-by-six 	{ margin-left: 56.6%;  }
	.row .offset-by-seven 	{ margin-left: 65.3%;  }
	.row .offset-by-eight 	{ margin-left: 74.0%;  }
	.row .offset-by-nine 	{ margin-left: 82.7%;  }
	.row .offset-by-ten 	{ margin-left: 91.4%;  }

	.row .centered { float: none; margin: 0 auto; }

	.row .offset-by-one:first-child 	{ margin-left: 8.7%;  }
	.row .offset-by-two:first-child 	{ margin-left: 17.4%;  }
	.row .offset-by-three:first-child 	{ margin-left: 26.1%;  }
	.row .offset-by-four:first-child 	{ margin-left: 34.8%;  }
	.row .offset-by-five:first-child 	{ margin-left: 43.5%;  }
	.row .offset-by-six:first-child 	{ margin-left: 52.2%;  }
	.row .offset-by-seven:first-child 	{ margin-left: 60.9%;  }
	.row .offset-by-eight:first-child 	{ margin-left: 69.6%;  }
	.row .offset-by-nine:first-child 	{ margin-left: 78.3%;  }
	.row .offset-by-ten:first-child 	{ margin-left: 87%;  }
	.row .offset-by-eleven:first-child 	{ margin-left: 95.7%;  }

	/* Source Ordering */
	.push-two 		{ left: 17.4% }
	.push-three 	{ left: 26.1%; }
	.push-four 		{ left: 34.8%; }
	.push-five 		{ left: 43.5%; }
	.push-six 		{ left: 52.2%; }
	.push-seven 	{ left: 60.9%; }
	.push-eight 	{ left: 69.6%; }
	.push-nine 		{ left: 78.3%; }
	.push-ten 		{ left: 87%; }

	.pull-two 		{ right: 17.4% }
	.pull-three 	{ right: 26.1%; }
	.pull-four 		{ right: 34.8%; }
	.pull-five 		{ right: 43.5%; }
	.pull-six 		{ right: 52.2%; }
	.pull-seven 	{ right: 60.9%; }
	.pull-eight 	{ right: 69.6%; }
	.pull-nine 		{ right: 78.3%; }
	.pull-ten 		{ right: 87%; }



	img, object, embed { max-width: 100%; height: auto; }
	img { -ms-interpolation-mode: bicubic; }
	#map_canvas img, .map_canvas img {max-width: none!important;}

	 /* Nicolas Gallagher's micro clearfix */
	 .row:before, .row:after, .cf:before, .cf:after { content:""; display:table; }
	 .row:after, .cf:after { clear: both; }
	 .row, .cf { zoom: 1; }


/*	--------------------------------------------------
	:: Block grids

	These are 2-up, 3-up, 4-up and 5-up ULs, suited
	for repeating blocks of content. Add 'mobile' to
	them to switch them just like the layout grid
	(one item per line) on phones

	For IE7/8 compatibility block-grid items need to be
	the same height. You can optionally uncomment the
	lines below to support arbitrary height, but know
	that IE7/8 do not support :nth-child.
	-------------------------------------------------- */

	.block-grid { display: block; overflow: hidden; }
	.block-grid>li { display: block; height: auto; float: left; }

	.block-grid.two-up { margin-left: -4% }
	.block-grid.two-up>li { margin-left: 4%; width: 46%; }
/* 	.block-grid.two-up>li:nth-child(2n+1) {clear: left;} */

	.block-grid.three-up { margin-left: -2% }
	.block-grid.three-up>li { margin-left: 2%; width: 31.3%; }
/* 	.block-grid.three-up>li:nth-child(3n+1) {clear: left;} */

	.block-grid.four-up { margin-left: -2% }
	.block-grid.four-up>li { margin-left: 2%; width: 23%; }
/* 	.block-grid.four-up>li:nth-child(4n+1) {clear: left;} */

	.block-grid.five-up { margin-left: -1.5% }
	.block-grid.five-up>li { margin-left: 1.5%; width: 18.5%; }
/* 	.block-grid.five-up>li:nth-child(5n+1) {clear: left;} */


/* --------------------------------------------------
   Table of Contents
-----------------------------------------------------
:: Buttons
:: Alerts
:: Labels
:: Tabs
:: Pagination
:: Breadcrumbs
:: Lists
:: Panels
:: Nav
:: Video
:: Microformats
*/


/*	--------------------------------------------------
	Buttons
	-------------------------------------------------- */

	.button {
		background: #00a6fc;
		display: inline-block;
		text-align: center;
		padding: 9px 34px 11px;
		color: #fff;
		text-decoration: none;
		font-weight: bold;
		font-size: 13px; font-size: 1.3rem;
		line-height: 1;
		font-family: "Helvetica Neue", "Helvetica", Arial, Verdana, sans-serif;
		position: relative;
		cursor: pointer;
		border: none;
		outline: none;
		margin: 0;
	}

  	/* Don't use native buttons on iOS */
	input[type=submit].button, button.button { -webkit-appearance: none; }

	/* Hide inner focus effect in Firefox */
	button::-moz-focus-inner, input[type="reset"]::-moz-focus-inner, input[type="button"]::-moz-focus-inner, input[type="submit"]::-moz-focus-inner, input[type="file"] > input[type="button"]::-moz-focus-inner {
		border: none;
	}

	.button.nice {
		background: #00a6fc url(../images/misc/button-gloss.png) repeat-x 0 -31px;
		-moz-box-shadow: inset 0 1px 0 rgba(255,255,255,.5);
		-webkit-box-shadow: inset 0 1px 0 rgba(255,255,255,.5);
		box-shadow: inset 0 1px 0 rgba(255,255,255,.5);
		text-shadow: 0 -1px 1px rgba(0,0,0,0.28);
		background: #00a6fc url(../images/misc/button-gloss.png) repeat-x 0 -31px, -moz-linear-gradient(top, rgba(255,255,255,.4) 0%, transparent 100%);
		background: #00a6fc url(../images/misc/button-gloss.png) repeat-x 0 -31px, -webkit-gradient(linear, left top, left bottom, color-stop(0%,rgba(255,255,255,.4)), color-stop(100%,transparent));
		border: 1px solid #0593dc;
		-webkit-transition: background-color .15s ease-in-out;
		-moz-transition: background-color .15s ease-in-out;
		-o-transition: background-color .15s ease-in-out;
	}

	.button.radius {
		-moz-border-radius: 3px;
		-webkit-border-radius: 3px;
		border-radius: 3px;
	}
	.button.round {
		-moz-border-radius: 1000px;
		-webkit-border-radius: 1000px;
		border-radius: 1000px;
	}

	.button.full-width {
		width: 100%;
		padding-left: 0 !important;
		padding-right: 0 !important;
		text-align: center;
	}

	.button.left-align {
		text-align: left;
		text-indent: 12px;
	}

	/* Sizes ---------- */
	.tiny.button			{ font-size: 10px; padding: 6px 14px 8px; width: auto; }
	.small.button 			{ font-size: 11px; padding: 8px 20px 10px; width: auto; }
	.medium.button 			{ font-size: 13px; width: auto; }
	.large.button 			{ font-size: 18px; padding: 11px 48px 13px; width: auto; }

	/* Nice Sizes ---------- */
	.nice.tiny.button 		{ background-position: 0 -37px; }
	.nice.small.button 		{ background-position: 0 -34px; }
	.nice.large.button 		{ background-position: 0 -30px; }

	/* Colors ---------- */
	.blue.button			{ background-color: #00a6fc; }
	.red.button				{ background-color: #e91c21; }
	.white.button			{ background-color: #e9e9e9; color: #333; }
	.black.button			{ background-color: #141414; }
	.green.button			{ background-color: #5b8737; }

	/* Nice Colors ---------- */
	.nice.blue.button		{ border: 1px solid #0593dc; }
	.nice.red.button		{ border: 1px solid #b90b0b; }
	.nice.white.button		{ border: 1px solid #cacaca; text-shadow: none !important; }
	.nice.black.button		{ border: 1px solid #000; }
	.nice.green.button		{ border: 1px solid #4a6b2d; }

	/* Hovers ---------- */
	.button:hover, .button:focus 				{ background-color: #0192dd; color: #fff; }
	.blue.button:hover, .blue.button:focus		{ background-color: #0192dd; }
	.red.button:hover, .red.button:focus 		{ background-color: #d01217; }
	.white.button:hover, .white.button:focus	{ background-color: #dadada; color: #333; }
	.black.button:hover, .black.button:focus	{ background-color: #000; }
	.green.button:hover, .green.button:focus	{ background-color: #699c40; }

	/* Hovers ---------- */
	.nice.button:active 	{
		-moz-box-shadow: inset 0 1px 0 rgba(0,0,0,.15);
		-webkit-box-shadow: inset 0 1px 0 rgba(0,0,0,.15);
		box-shadow: inset 0 1px 0 rgba(0,0,0,.15);
	}

	/* Disabled ---------- */
	.button.disabled, .button[disabled] { opacity: 0.6; cursor: default; }

	/* Correct FF button padding */
	@-moz-document url-prefix(http://) {
		input[type=submit].button::-moz-focus-inner, button.button::-moz-focus-inner { border: 0; padding: 0; }
		input[type=submit].tiny.button		{ padding: 5px 14px 7px; }
		input[type=submit].small.button		{ padding: 7px 20px 8px; }
		input[type=submit].medium.button	{ padding: 8px 34px 9px; }
		input[type=submit].large.button		{ padding: 9px 48px 10px; }
	}

/*	--------------------------------------------------
	Alerts
	-------------------------------------------------- */

	div.alert-box { display: block; padding: 6px 7px; font-weight: bold; font-size: 13px; background: #eee; border: 1px solid rgba(0,0,0,0.1); margin-bottom: 12px; border-radius: 3px; -webkit-border-radius: 3px; -moz-border-radius: 3px; text-shadow: 0 1px rgba(255,255,255,0.9); position: relative; }
	.alert-box.success { background-color: #7fae00; color: #fff; text-shadow: 0 -1px rgba(0,0,0,0.3); }
	.alert-box.warning { background-color: #f68b01; color: #fff; text-shadow: 0 -1px rgba(0,0,0,0.3); }
	.alert-box.error { background-color: #c00000; color: #fff; text-shadow: 0 -1px rgba(0,0,0,0.3); }

	.alert-box a.close { color: #000; position: absolute; right: 4px; top: 0; font-size: 18px; opacity: 0.2; padding: 4px; }
	.alert-box a.close:hover,.alert-box a.close:focus { opacity: 0.4; }

/*  --------------------------------------------------
    Pagination
    -------------------------------------------------- */
    ul.pagination { display: block; height: 24px; margin-left: -5px; }
    ul.pagination li { float: left; display: block; height: 24px; color: #999; font-size: 15px; margin-left: 5px; }
    ul.pagination li a { display: block; padding: 6px 7px 4px; color: #555; }
    ul.pagination li.current a, ul.pagination li:hover a, ul.pagination li a:focus { border-bottom: solid 2px #00a6fc; color: #141414; }
    ul.pagination li.unavailable a { cursor: default; color: #999; }
    ul.pagination li.unavailable:hover a, ul.pagination li.unavailable a:focus { border-bottom: none; }

/*  --------------------------------------------------
    Breadcrumbs, SubNav
    -------------------------------------------------- */
    ul.breadcrumbs { display: block; background: #f4f4f4; padding: 7px 10px; border: 1px solid #ddd; border-radius: 2px; overflow: hidden; }
	ul.breadcrumbs li { margin: 0; padding: 0 12px 0 0; float: left; }
	ul.breadcrumbs li a, ul.breadcrumbs li span { text-transform: uppercase; font-size: 11px; padding-left: 12px; }
	ul.breadcrumbs li:first-child a, ul.breadcrumbs li:first-child span { padding-left: 0; }
	ul.breadcrumbs li:before { content: "/"; color: #aaa; }
	ul.breadcrumbs li:first-child:before { content: ""; }
	ul.breadcrumbs li.current a { cursor: default; color: #333; }
	ul.breadcrumbs li:hover a, ul.breadcrumbs li a:focus { text-decoration: underline; }
	ul.breadcrumbs li.current:hover a, ul.breadcrumbs li.current a:focus { text-decoration: none; }
	ul.breadcrumbs li.unavailable a { color: #999; }
    ul.breadcrumbs li.unavailable:hover a, ul.pagination li.unavailable a:focus { text-decoration: none; color: #999; cursor: default; }

	dl.sub-nav{display:block;width:auto;overflow:hidden;padding-top:4px;margin:0 0 10px -9px;font-size:11px;font-weight:700}
	dl.sub-nav dt,dl.sub-nav dd{float:left;display:inline;margin-left:9px;margin-bottom:4px}
	dl.sub-nav dt{color:#999;font-weight:400}
	dl.sub-nav dd a{text-decoration:none;-webkit-border-radius:12px;-moz-border-radius:12px}
	dl.sub-nav dd.active a{background:#00A6FC;color:#fff;padding:3px 9px}

/*  --------------------------------------------------
    Lists
    -------------------------------------------------- */
    ul.nice, ol.nice { list-style: none; margin: 0; }
    ul.nice li, ol.nice li { padding-left: 13px; position: relative }
    ul.nice li span.bullet, ol.nice li span.number { position: absolute; left: 0; top: 0; color: #ccc; }

/*	--------------------------------------------------
	Panels
	-------------------------------------------------- */
	div.panel {
		padding: 20px 20px 2px 20px;
		background: #efefef;
		background: -moz-linear-gradient(top, #FFFFFF 0%, #F4F4F4 100%);
		background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#FFFFFF), color-stop(100%,#F4F4F4));
		background: -o-linear-gradient(top, #ffffff 0%,#f4f4f4 100%);
		filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#FFFFFF', endColorstr='#F4F4F4',GradientType=0 );
		box-shadow: 0 2px 5px rgba(0,0,0,0.15);
		-webkit-box-shadow: 0 2px 5px rgba(0,0,0,0.15);
		-moz-box-shadow: 0 2px 5px rgba(0,0,0,0.25);
		margin: 0 0 20px 0;
	}

/*	--------------------------------------------------
   	Microformats
	-------------------------------------------------- */

	/* hCard */
	ul.vcard { display: inline-block; margin: 0 0 12px 0; border: 1px solid #ddd; padding: 10px; }
	ul.vcard li { margin: 0; display: block; }
	ul.vcard li.fn { font-weight: bold; font-size: 15px; font-size: 1.5rem; }

	p.vevent span.summary { font-weight: bold; }
	p.vevent abbr { cursor: default; text-decoration: none; font-weight: bold; border: none; padding: 0 1px; }

/* -----------------------------------------
   Standard Forms
----------------------------------------- */

	form { margin: 0 0 18px; }
	form label { display: block; font-size: 13px; line-height: 18px; cursor: pointer; margin-bottom: 9px; }

	input.input-text, textarea { border-right: 1px solid #bbb; border-bottom: 1px solid #bbb; }
	input.input-text, textarea, select { display: block; margin-bottom: 9px; }
	label + input.input-text, label + textarea, label + select, label + div.dropdown, select + div.dropdown { margin-top: -9px; }

	/* Text input and textarea font and padding */
	input.input-text, textarea { font-size: 13px; padding: 4px 3px 2px; background: #fff; }
	input.input-text:focus, textarea:focus { outline: none !important; }
	input.input-text.oversize, textarea.oversize { font-size: 18px !important; padding: 4px 5px !important; }
	input.input-text:focus, textarea:focus { background: #f9f9f9; }

	/* Text input and textarea, disabled */
	input.input-text[disabled], textarea[disabled] { background-color: #ddd; }

	/* Inlined Label Style */
	input.placeholder, textarea.placeholder { color: #888; }

	/* Text input and textarea sizes */
	input.input-text, textarea { width: 254px; box-sizing: border-box; -webkit-box-sizing: border-box; -moz-box-sizing: border-box; }
	input.small, textarea.small { width: 134px; }
	input.medium, textarea.medium { width: 254px; }
	input.large, textarea.large { width: 434px; }
	input.expand, textarea.expand { width: 100%; }

	/* Fieldsets */
	form fieldset { padding: 9px 9px 2px 9px; border: solid 1px #ddd; margin: 18px 0; }

	/* Inlined Radio & Checkbox */
	.form-field input[type=radio], div.form-field input[type=checkbox] { display: inline; width:auto; margin-bottom:0; }

	/* Errors */
	.form-field.error input, input.input-text.red,.form-field.error textarea { border-color: #C00000; background-color: rgba(255,0,0,0.15); }
	.form-field.error label, label.red { color: #C00000; }
	.form-field.error small, small.error { margin-top: -6px; display: block; margin-bottom: 9px; font-size: 11px; color: #C00000; width: 254px; box-sizing: border-box; -webkit-box-sizing: border-box; -moz-box-sizing: border-box; }

	.small + small.error { width: 134px; }
	.medium + small.error { width: 254px; }
	.large + small.error { width: 434px; }
	.expand + small.error { width: 100%; }

	.small.oversize + small.error { width: 144px; }
	.medium.oversize + small.error { width: 264px; }
	.large.oversize + small.error { width: 444px; }
	.expand.oversize + small.error { width: 100%; }

	/* -----------------------------------------
	   Nicer Forms
	----------------------------------------- */
	form.nice div.form-field input, form.nice input.input-text, form.nice textarea { border: solid 1px #bbb; border-radius: 2px; -webkit-border-radius: 2px; -moz-border-radius: 2px; }
	form.nice div.form-field input, form.nice input.input-text, form.nice textarea { font-size: 13px; padding: 6px 3px 4px; outline: none !important; background: url(../images/misc/input-bg.png) #fff; }
	form.nice div.form-field input:focus, form.nice input.input-text:focus, form.nice textarea:focus { background-color: #f9f9f9; }

	/* Text input and textarea, disabled */
	form.nice div.form-field input[disabled], form.nice input.input-text[disabled], form.nice textarea[disabled] { background-color: #ddd; }

	form.nice fieldset { border-radius: 3px; -webkit-border-radius: 3px; -moz-border-radius: 3px; }

	form.nice div.form-field input[type=radio], form.nice div.form-field input[type=checkbox] { display: inline; width:auto; margin-bottom:0; }

	form.nice div.form-field.error small, form.nice small.error { padding: 6px 4px; border: solid 0 #C00000; border-width: 0 1px 1px 1px; margin-top: -10px; background: #C00000; color: #fff; font-size: 12px; font-weight: bold; border-bottom-left-radius: 2px; border-bottom-right-radius: 2px; -webkit-border-bottom-left-radius: 2px; -webkit-border-bottom-right-radius: 2px; -moz-border-radius-bottomleft: 2px; -moz-border-radius-bottomright: 2px; }

 	form.nice div.form-field.error .small + small, form.nice .small + small.error { width: 134px; }
	form.nice div.form-field.error .medium + small, form.nice .medium + small.error { width: 254px; }
	form.nice div.form-field.error .large + small, form.nice .large + small.error { width: 434px; }

	form.nice div.form-field.error .small.oversize + small, form.nice .small.oversize + small.error { width: 136px; }
	form.nice div.form-field.error .medium.oversize + small, form.nice .medium.oversize + small.error { width: 256px; }
	form.nice div.form-field.error .large.oversize + small, form.nice .large.oversize + small.error { width: 436px; }

	/* -----------------------------------------
	   Custom Forms
	----------------------------------------- */

	form.custom span.custom { display: inline-block; width: 14px; height: 14px; position: relative; top: 2px; border: solid 1px #ccc; background: url(../images/misc/custom-form-sprites.png) 0 0 no-repeat; }
	form.custom span.custom.radio { border-radius: 7px; -webkit-border-radius: 7px; -moz-border-radius: 7px; }
	form.custom span.custom.radio.checked { background-position: 0 -14px; }
	form.custom span.custom.checkbox.checked { background-position: 0 -28px; }

	form.custom div.custom.dropdown { display: block; position: relative; width: auto; height: 28px; margin-bottom: 9px; margin-top: 2px; }
	form.custom div.custom.dropdown a.current { display: block; width: auto; line-height: 26px; min-height: 26px; padding: 0 38px 0 6px; border: solid 1px #ddd; color: #141414; background-color: #fff; word-wrap: nowrap }
	form.custom div.custom.dropdown a.selector { position: absolute; width: 26px; height: 26px; display: block; background: url(../images/misc/custom-form-sprites.png) -14px 1px no-repeat; right: 0; top: 0; border: solid 1px #ddd; }
	form.custom div.custom.dropdown:hover a.selector,
	form.custom div.custom.dropdown.open a.selector { background-position: -14px -26px; }

	form.custom div.custom.dropdown ul { position: absolute; width: auto; display: none; margin: 0; left: 0; top: 27px; margin: 0; padding: 0; background: #fff; background: rgba(255,255,255,0.95); border: solid 1px #ddd; }
	form.custom div.custom.dropdown ul li { cursor: pointer; padding: 3px 38px 3px 6px; min-height: 18px; margin: 0; white-space: nowrap}
	form.custom div.custom.dropdown ul li.selected { background: url(../images/misc/custom-form-sprites.png) right -52px no-repeat; }
	form.custom div.custom.dropdown ul li:hover { background-color: #2a85e8; color: #fff; }
	form.custom div.custom.dropdown ul li.selected:hover { background: url(../images/misc/custom-form-sprites.png) #2a85e8 right -78px no-repeat; }
	form.custom div.custom.dropdown ul.show { display: block; }

	form.custom div.custom.dropdown.open ul { display: block; z-index: 10; }

	form.custom div.custom.dropdown.small { width: 134px !important; }
	form.custom div.custom.dropdown.medium { width: 254px !important; }
	form.custom div.custom.dropdown.large { width: 434px !important; }
	form.custom div.custom.dropdown.expand { width: 100% !important; }

	form.custom div.custom.dropdown.open.small ul { width: 132px !important; }
	form.custom div.custom.dropdown.open.medium ul { width: 252px !important; }
	form.custom div.custom.dropdown.open.large ul { width: 432px !important; }
	form.custom div.custom.dropdown.open.expand ul { width: 100% !important; box-sizing: border-box; }

	/* Custom input, disabled */
	form.custom .custom.disabled { background-color: #ddd; }

	/* -----------------------------------------
	   Nicer Custom Forms
	----------------------------------------- */
	form.custom.nice span.custom.checkbox { border-radius: 2px; -webkit-border-radius: 2px; -moz-border-radius: 2px; }

	form.custom.nice div.custom.dropdown a.current { border-radius: 2px; -webkit-border-radius: 2px; -moz-border-radius: 2px; background-image: url(../images/misc/input-bg-outset.png); background-position: bottom right; }
	form.custom.nice div.custom.dropdown a.selector { height: 27px; border-width: 0 0 0 1px; }
	form.custom.nice div.custom.dropdown ul { border-bottom-left-radius: 2px; border-bottom-right-radius: 2px; -webkit-border-bottom-left-radius: 2px; -webkit-border-bottom-right-radius: 2px; -moz-border-radius-bottomleft: 2px; -moz-border-radius-bottomright: 2px; }

/* --------------------------------------------------
	:: Typography
	-------------------------------------------------- */

	@media handheld, only screen and (max-width: 767px) {
		h1 { font-size: 32px; font-size: 3.2rem; line-height: 1.3; }
		h2 { font-size: 28px; font-size: 2.8rem; line-height: 1.3; }
		h3 { font-size: 21px; font-size: 2.1rem; line-height: 1.3; }
		h4 { font-size: 18px; font-size: 1.8rem; line-height: 1.2; }
		h5 { font-size: 16px; font-size: 1.6rem; line-height: 1.2; }
		h6 { font-size: 15px; font-size: 1.5rem; line-height: 1.2; }
		body, p {  font-size: 15px; font-size: 1.5rem; line-height: 1.4; }
	}

/* --------------------------------------------------
	:: Grid
	-------------------------------------------------- */

	/* Mobile */
	@media only screen and (max-width: 767px) {
		body { -webkit-text-size-adjust: none; -ms-text-size-adjust: none; width: 100%; min-width: 0; margin-left: 0; margin-right: 0; padding-left: 0; padding-right: 0; }
		.container { min-width: 0; margin-left: 0; margin-right: 0; }
		.row { width: 100%; min-width: 0; margin-left: 0; margin-right: 0; }
		.row .row .column, .row .row .columns { padding: 0; }
		.column, .columns { width: auto !important; float: none; margin-left: 0; margin-right: 0; }
		.column:last-child, .columns:last-child { margin-right: 0; float: none; }
		[class*="column"] + [class*="column"]:last-child { float: none; }
		[class*="column"]:before, [class*="column"]:after { content:""; display:table; }
		[class*="column"]:after { clear: both; }

		.offset-by-one, .offset-by-two, .offset-by-three, .offset-by-four, .offset-by-five, .offset-by-six, .offset-by-seven, .offset-by-eight, .offset-by-nine, .offset-by-ten, .offset-by-eleven, .centered { margin-left: 0 !important; }

		.push-two, .push-three, .push-four, .push-five, .push-six, .push-seven, .push-eight, .push-nine, .push-ten { left: auto; }
		.pull-two, .pull-three, .pull-four, .pull-five, .pull-six, .pull-seven, .pull-eight, .pull-nine, .pull-ten { right: auto; }

		/* Mobile 4-column Grid */
		.row .phone-one:first-child, .row .phone-two:first-child, .row .phone-three:first-child, .row .phone-four:first-child { margin-left: 0; }
		.row .phone-one:last-child, .row .phone-two:last-child, .row .phone-three:last-child, .row .phone-four:last-child { float: right; }

		.row .phone-one, .row .phone-two, .row .phone-three, .row .phone-four { margin-left: 4.4%; float: left; min-height: 1px; position: relative; padding: 0; }

		.row .phone-one 	{ width: 21.68% !important; }
		.row .phone-two 	{ width: 47.8% !important; }
		.row .phone-three 	{ width: 73.9% !important; }
		.row .phone-four 	{ width: 100% !important; }

		.row .push-one-phone { left: 26.08%; }
		.row .push-two-phone { left: 52.2% }
		.row .push-three-phone { left: 78.3% }

		.row .pull-one-phone { right: 26.08% }
		.row .pull-two-phone { right: 52.2% }
		.row .pull-three-phone { right: 78.3%; }
	}

/* --------------------------------------------------
	:: Block Grids
	-------------------------------------------------- */

	@media only screen and (max-width: 767px) {
		.block-grid.mobile { margin-left: 0; }
		.block-grid.mobile > li { float: none; width: 100%; margin-left: 0; }
	}

/* --------------------------------------------------
	:: Mobile Visibility Affordances
---------------------------------------------------*/

	.show-on-phones { display: none !important; }
	.show-on-tablets { display: none !important; }
	.show-on-desktops { display: block !important; }

	.hide-on-phones { display: block !important; }
	.hide-on-tablets { display: block !important; }
	.hide-on-desktops { display: none !important; }


	/* Modernizr-enabled tablet targeting */
	@media only screen and (max-width: 1280px) and (min-width: 768px) {
		.touch .hide-on-phones { display: block !important; }
		.touch .hide-on-tablets { display: none !important; }
		.touch .hide-on-desktops { display: block !important; }

		.touch .show-on-phones { display: none !important; }
		.touch .show-on-tablets { display: block !important; }
		.touch .show-on-desktops { display: none !important; }
	}


	@media only screen and (max-width: 767px) {
		.hide-on-phones { display: none !important; }
		.hide-on-tablets { display: block !important; }
		.hide-on-desktops { display: block !important; }

		.show-on-phones { display: block !important; }
		.show-on-tablets { display: none !important; }
		.show-on-desktops { display: none !important; }
	}

	/* Specific overrides for elements that require something other than display: block */

	table.show-on-desktops { display: table !important; }
	table.hide-on-phones { display: table !important; }
	table.hide-on-tablets { display: table !important; }

	@media only screen and (max-width: 1280px) and (min-width: 768px) {
		.touch table.hide-on-phones { display: table !important; }
		.touch table.hide-on-desktops { display: table !important; }
		.touch table.show-on-tablets { display: table !important; }
	}

	@media only screen and (max-width: 767px) {
		table.hide-on-tablets { display: table !important; }
		table.hide-on-desktops { display: table !important; }
		table.show-on-phones { display: table !important; }
	}


/* --------------------------------------------------
	:: Forms
---------------------------------------------------*/

	@media only screen and (max-width: 767px) {
		div.form-field input, div.form-field input.small, div.form-field input.medium, div.form-field input.large, div.form-field input.oversize, input.input-text, input.input-text.oversize, textarea,
		form.nice div.form-field input, form.nice div.form-field input.oversize, form.nice input.input-text, form.nice input.input-text.oversize, form.nice textarea { display: block; width: 100%; padding: 6px 2% 4px; font-size: 18px; }
		form.nice div.form-field input, form.nice div.form-field input.oversize, form.nice input.input-text, form.nice input.input-text.oversize, form.nice textarea { -webkit-border-radius: 2px; -moz-border-radius: 2px; border-radius: 2px; }
		form.nice div.form-field.error small, form.nice small.error { padding: 6px 2%; display: block; }
		form.nice div.form-field.error .small + small, form.nice .small + .error { width: auto; }
		form.nice div.form-field.error .medium + small, form.nice .medium + .error { width: auto; }
		form.nice div.form-field.error .large + small, form.nice .large + .error { width: auto; }
		form.nice div.form-field.error .expand + small, form.nice .expand + .error { width: auto; }
	}


/* --------------------------------------------------
	:: UI
---------------------------------------------------*/

	/* Buttons */
	@media only screen and (max-width: 767px) {
		.button { display: block; }
		button.button, input[type="submit"].button { width: 100%; padding-left: 0; padding-right: 0; margin-bottom: 5px; }
	}
